@@ -1,6 +1,19 @@
+#  Licensed under the Apache License, Version 2.0 (the "License"); you may
+#  not use this file except in compliance with the License. You may obtain
+#  a copy of the License at
+#
+#       http://www.apache.org/licenses/LICENSE-2.0
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+#  License for the specific language governing permissions and limitations
+#  under the License.
+
 """Discover and lookup command plugins.
 """
 
+import inspect
 import logging
 
 import pkg_resources
@@ -17,7 +30,7 @@ class EntryPointWrapper(object):
         self.name = name
         self.command_class = command_class
 
-    def load(self):
+    def load(self, require=False):
         return self.command_class
 
 
@@ -37,7 +50,7 @@ class CommandManager(object):
         self._load_commands()
 
     def _load_commands(self):
-        # NOTE(jamielennox): kept for compatability.
+        # NOTE(jamielennox): kept for compatibility.
         self.load_commands(self.namespace)
 
     def load_commands(self, namespace):
@@ -60,18 +73,32 @@ class CommandManager(object):
         """Given an argument list, find a command and
         return the processor and any remaining arguments.
         """
-        search_args = argv[:]
-        name = ''
-        while search_args:
-            if search_args[0].startswith('-'):
-                name = '%s %s' % (name, search_args[0])
-                raise ValueError('Invalid command %r' % name)
-            next_val = search_args.pop(0)
-            name = '%s %s' % (name, next_val) if name else next_val
+        start = self._get_last_possible_command_index(argv)
+        for i in range(start, 0, -1):
+            name = ' '.join(argv[:i])
+            search_args = argv[i:]
             if name in self.commands:
                 cmd_ep = self.commands[name]
-                cmd_factory = cmd_ep.load()
+                if hasattr(cmd_ep, 'resolve'):
+                    cmd_factory = cmd_ep.resolve()
+                else:
+                    # NOTE(dhellmann): Some fake classes don't take
+                    # require as an argument. Yay?
+                    arg_spec = inspect.getargspec(cmd_ep.load)
+                    if 'require' in arg_spec[0]:
+                        cmd_factory = cmd_ep.load(require=False)
+                    else:
+                        cmd_factory = cmd_ep.load()
                 return (cmd_factory, name, search_args)
         else:
             raise ValueError('Unknown command %r' %
                              (argv,))
+
+    def _get_last_possible_command_index(self, argv):
+        """Returns the index after the last argument
+        in argv that can be a command word
+        """
+        for i, arg in enumerate(argv):
+            if arg.startswith('-'):
+                return i
+        return len(argv)
